/****************************************************************************
*  Copyright (C) 2008-2012 by Michael Fischer.
*
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
****************************************************************************
*  History:
*
*  07.11.2008  mifi  First Version, based on FatFs example.
*  11.02.2012  mifi  Tested with EIR.
*  23.08.2012  mifi  Tested with an Altera DE1.
****************************************************************************/
#define __MAIN_C__

/*=========================================================================*/
/*  Includes                                                               */
/*=========================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include <system.h>
#include <sys/alt_alarm.h>
#include <io.h>

#include "./sd_card/fatfs.h"
#include "./sd_card/diskio.h"

#include "./sd_card/ff.h"
#include "./sd_card/monitor.h"

#include "./uart/uart.h"

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Macros                                                         */
/*=========================================================================*/

#define PSTR(_a)  _a

/*=========================================================================*/
/*  DEFINE: Prototypes                                                     */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/
static alt_alarm alarm2;
static unsigned long Systick = 0;
static volatile unsigned short Timer;   /* 1000Hz increment timer */

/*=========================================================================*/
/*  DEFINE: Definition of all local Procedures                             */
/*=========================================================================*/

/***************************************************************************/
/*  TimerFunction                                                          */
/*                                                                         */
/*  This timer function will provide a 10ms timer and                      */
/*  call ffs_DiskIOTimerproc.                                              */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static alt_u32 TimerFunction (void *context)
{
   static unsigned short wTimer10ms = 0;
   
   (void)context;

   Systick++;
   wTimer10ms++;
   Timer++; /* Performance counter for this module */

   if (wTimer10ms == 10) 
   {
      wTimer10ms = 0;
      ffs_DiskIOTimerproc();  /* Drive timer procedure of low level disk I/O module */
   }

   return(1);
} /* TimerFunction */

/***************************************************************************/
/*  IoInit                                                                 */
/*                                                                         */
/*  Init the hardware like GPIO, UART, and more...                         */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void IoInit(void)
{
printf("inicializando DISK IO\n");
   /* Init diskio interface */
   ffs_DiskIOInit();
printf("inicializando TASK IO FUC\n");
   /* Init timer system */
   alt_alarm_start(&alarm2, 1, &TimerFunction, NULL);

} /* IoInit */

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/


FATFS Fatfs[_VOLUMES];          /* File system object for each logical drive */
FIL File1, File2;               /* File objects */
DIR Dir;                        /* Directory object */
uint8_t Buff[8192] __attribute__ ((aligned(4)));  /* Working buffer */



/***************************************************************************/
/*  main                                                                   */
/***************************************************************************/
int main(void)
{


    FATFS *fs;                  /* Pointer to file system object */


    // Inicializacin de drivers del sistema

    // inicializacin SD driver
    IoInit();
    printf("rc=%d\n", (uint16_t) disk_initialize(0));
    f_mount(0, &Fatfs[0]);


    //inicializacin UART
      InitUart(38400);


    while(1){// programa principal


       //Este es el codigo en python que ustede debe pasar a C
       // y agregar ademas el control de los puertos IO para seleccionar la imagen .nes que se va a
       // programar por medio del puerto UART

      /*
        #which rom do you want to program on FPGA?
        rom=open('roms/ece31289upb.nes','rb');
        header=rom.read(4);
        print header
        if(header=='NES\x1a'):
            print 'Reading valid rom';
        else:
            print 'Not valid NES ROM.. Sorry';
            exit(1)


        progRomBanks= rom.read(1);
        chrRomBanks= rom.read(1);
        progRomBanks=ord(progRomBanks)
        chrRomBanks=ord(chrRomBanks)

        print 'progRomBanks=' + str(progRomBanks)
        print 'chrRomBanks=' + str(progRomBanks)


        if(progRomBanks>SupportedProgRomBanks or chrRomBanks>SupportedChrRomBanks):
            print 'Too many ROM Banks\nYou should try yo expand the memory to UxROM on the FPGA'
            exit(1)

        Flag6=ord(rom.read(1));
        Flag7=ord(rom.read(1));
        Flag7_1=ord(rom.read(1));

        if (Flag6 & 0x08):
            print "Only horizontal and vertical mirroring are supported."
            exit(1)

        if(Flag6 & 0x04):
            print 'SRAM at 6000-7FFFh battery backed.  Yes'
        else:
            print 'SRAM at 6000-7FFFh battery backed.  No'

        if(Flag6 & 0x02):
            print '512 byte trainer at 7000-71FFh'
        else:
            print 'no trainer present'

        if(Flag6 & 0x01):
            print 'Four screen mode Yes'
        else:
            print 'Four screen mode No'

        if ((((Flag6 & 0xF0) >> 4)| (Flag7 & 0xF0)) != 0):
            print "Only mapper 0 is supported.";
            #exit(1)

        #debugger break (stop execution)
        send_byte(DbgPacketOpCodeDbgHlt);


        #disable PPU
        send_byte(DbgPacketOpCodePpuDisable);

        #Set iNES header info to configure mappers.
        #send    DbgPacketOpCodePpuDisable
        #send    progRomBanks
        #send    chrRomBanks
        #send    Flag6
        #send    Flag7
        #send    Flag7_1

        print 'Set iNES header info to configure mappers.'
        send_byte(DbgPacketOpCodeCartSetCfg);
        send_byte(progRomBanks);
        send_byte(chrRomBanks);
        send_byte(Flag6);
        send_byte(Flag7);
        send_byte(Flag7_1);


        #math size of banks
        prgRomDataSize    = progRomBanks * 0x4000;
        chrRomDataSize    = chrRomBanks * 0x2000;
        totalBytes        = prgRomDataSize + chrRomDataSize;
        transferBlockSize = 0x400;


        #copy PRG ROM data
        _i=0
        prgRomOffset=0;
        while(_i<(prgRomDataSize / transferBlockSize)):
            prgRomOffset = transferBlockSize * _i;
            rom.seek(16+prgRomOffset)
            m_pData=''
            m_pData = m_pData +chr(DbgPacketOpCodeCpuMemWr);
            m_pData = m_pData + chr((0x8000 + prgRomOffset) & 0xff);
            m_pData = m_pData +chr(((0x8000 + prgRomOffset) & 0xff00)>>8);
            m_pData = m_pData +chr(transferBlockSize & 0xff);
            m_pData = m_pData +chr((transferBlockSize & 0xff00)>>8);
            send_data=rom.read(transferBlockSize);
            m_pData=m_pData+send_data
            ser.flushOutput()
            ser.write(m_pData)
            print '->Programming PRG ROM ADDR ' + hex(0x8000 + prgRomOffset)
            _i=_i+1;


        #copy CHR ROM data
        _i=0;
        while(_i<(chrRomDataSize / transferBlockSize)):
            chrRomOffset = transferBlockSize * _i;
            rom.seek(16+prgRomDataSize+chrRomOffset)
            m_pData=''
            m_pData = m_pData +chr(DbgPacketOpCodePpuMemWr);
            m_pData = m_pData + chr((chrRomOffset) & 0xff);
            m_pData = m_pData +chr(((chrRomOffset) & 0xff00)>>8);
            m_pData = m_pData +chr(transferBlockSize & 0xff);
            m_pData = m_pData +chr((transferBlockSize & 0xff00)>>8);
            send_data=rom.read(transferBlockSize);
            m_pData=m_pData+send_data
            ser.flushOutput();
            ser.write(m_pData)
            print '->Programming CHR ROM ADDR ' + hex(chrRomOffset)
            _i=_i+1;

        # Update PC to point at the reset interrupt vector location.
        rom.seek(16+prgRomDataSize - 4)
        pclVal = rom.read(1);
        rom.seek(16 + prgRomDataSize - 3)
        pchVal = rom.read(1);

        print 'Update PC to point at the reset interrupt vector location'
        m_pData ='';
        m_pData = m_pData+ chr(DbgPacketOpCodeCpuRegWr);
        m_pData = m_pData+ chr(CpuRegPcl);
        m_pData = m_pData+ pclVal;
        ser.write(m_pData)


        m_pData ='';
        m_pData = m_pData+ chr(DbgPacketOpCodeCpuRegWr);
        m_pData = m_pData+ chr(CpuRegPch);
        m_pData = m_pData+ pchVal;
        ser.write(m_pData)

        # Issue a debug run command.
        print 'sending run command'
        send_byte(DbgPacketOpCodeDbgRun);

        rom.close()
        ser.close()

      */


    }


    return (0);
}

/*** EOF ***/


